//
//  BMKBackgroundLocController.m
//  BMKBackgroundLocOCDemo
//
//  Created by v_chiyuanwei on 2020/7/15.
//  Copyright © 2020 baidu. All rights reserved.
//

#import "BMKBackgroundLocController.h"

#import <BaiduMapAPI_Map/BMKMapComponent.h>
#import <BMKLocationkit/BMKLocationComponent.h>

#import "BMKBackgroundLocTabView.h"
#import "BMKBackgroundLocInfoView.h"

@interface BMKBackgroundLocController () <BMKLocationManagerDelegate>
/// 地图
@property (nonatomic, strong) BMKMapView *mapView;
/// 定位按钮
@property (nonatomic, strong) BMKBackgroundLocTabView *locTabView;
/// 定位结果
@property (nonatomic, strong) BMKBackgroundLocInfoView *locInfoView;
/// 定位管理类
@property(nonatomic, strong) BMKLocationManager *locationManager;

@end

@implementation BMKBackgroundLocController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.title = @"后台持续定位";
    
    [self initialize];
    [self setupViews];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    // 当mapView即将被显示的时候调用，恢复之前存储的mapView状态
    [self.mapView viewWillAppear];
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    // 当mapView即将被隐藏的时候调用，存储当前mapView的状态
    [self.mapView viewWillDisappear];
}

#pragma mark -
- (void)initialize {
    self.view.backgroundColor = [UIColor whiteColor];
    self.navigationController.navigationBar.translucent = NO;
    self.navigationController.navigationBar.barStyle = UIBarStyleBlack;
    self.navigationController.navigationBar.barTintColor = [UIColor colorWithRed:34.f / 255.f green:37.f / 255.f blue:61.f / 255.f alpha:1.f];
    [self.navigationController.navigationBar setTitleTextAttributes:@{NSForegroundColorAttributeName:[UIColor whiteColor]}];
}

- (void)setupViews {
    [self.view addSubview:self.mapView];
    [self.view addSubview:self.locTabView];
    [self.view addSubview:self.locInfoView];
}

/// 更新我的位置数据
- (void)updateUserLocation:(BMKLocation *)location {
    BMKUserLocation *userLocation = [[BMKUserLocation alloc] init];
    userLocation.location = location.location;
    // 如需展示我的位置图标需要调用此方法
    [self.mapView updateLocationData:userLocation];
    // 设置地图中心点
    [self.mapView setCenterCoordinate:location.location.coordinate];
}

#pragma mark - <BMKLocationManagerDelegate>
///为了适配app store关于新的后台定位的审核机制（app  store要求如果开发者只配置了使用期间定位，则代码中不能出现申请后台定位的逻辑），
///当开发者在plist配置NSLocationAlwaysUsageDescription或者NSLocationAlwaysAndWhenInUseUsageDescription时，
///需要在该delegate中调用后台定位api：[locationManager requestAlwaysAuthorization]。
///开发者如果只配置了NSLocationWhenInUseUsageDescription，且只有使用期间的定位需求，则无需在delegate中实现逻辑。
- (void)BMKLocationManager:(BMKLocationManager * _Nonnull)manager doRequestAlwaysAuthorization:(CLLocationManager * _Nonnull)locationManager {
    [locationManager requestAlwaysAuthorization];
}

/// 连续定位回调函数
- (void)BMKLocationManager:(BMKLocationManager *)manager didUpdateLocation:(BMKLocation *)location orError:(NSError *)error {
    // 定位错误信息
    if (error) {
        NSLog(@"locError:{%zd - %@};", error.code, error.localizedDescription);
        return;
    }

    // 定位结果
    if (location) {
        // 后台定位经纬度展示
        CLLocationCoordinate2D coord = location.location.coordinate;
        NSString *coordStr = [NSString stringWithFormat:@"纬度：%f，经度：%f", coord.latitude, coord.longitude];
        
        if (self.locInfoView.locInfo) {
            self.locInfoView.locInfo = [NSString stringWithFormat:@"%@\n%@", self.locInfoView.locInfo, coordStr];
        } else {
            self.locInfoView.locInfo = [NSString stringWithFormat:@"%@", coordStr];
        }
        
        // 更新我的位置数据
        [self updateUserLocation:location];
    }
}

/// 当定位发生错误时，会调用代理的此方法。
- (void)BMKLocationManager:(BMKLocationManager * _Nonnull)manager didFailWithError:(NSError * _Nullable)error {
    NSLog(@"serial loc error = %@", error);
}

/// 定位权限状态改变时回调函数
- (void)BMKLocationManager:(BMKLocationManager * _Nonnull)manager didChangeAuthorizationStatus:(CLAuthorizationStatus)status {
    NSLog(@"serial loc CLAuthorizationStatus = %d", status);
}

#pragma mark - getter
/// 地图
- (BMKMapView *)mapView {
    if (!_mapView) {
        _mapView = [[BMKMapView alloc] initWithFrame: CGRectMake(0, 0, SCREEN_WIDTH, CONTENT_HEIGHT)];
        // 设定是否显示定位图层
        _mapView.showsUserLocation = YES;
    }
    return _mapView;
}

/// 单次定位管理类
- (BMKLocationManager *)locationManager {
    if (!_locationManager) {
        _locationManager = [[BMKLocationManager alloc] init];
        // 设置返回位置的坐标系类型
        _locationManager.coordinateType = BMKLocationCoordinateTypeBMK09LL;
        // 设置距离过滤参数
        _locationManager.distanceFilter = kCLDistanceFilterNone;
        // 设置预期精度参数
        _locationManager.desiredAccuracy = kCLLocationAccuracyBest;
        // 设置应用位置类型
        _locationManager.activityType = CLActivityTypeAutomotiveNavigation;
        // 设置是否自动停止位置更新
        _locationManager.pausesLocationUpdatesAutomatically = NO;
        //设置是否允许后台定位, 记得在TARGETS->Capabilities->Background Modes中勾选 Location updates
        _locationManager.allowsBackgroundLocationUpdates = YES;
        // 设置位置获取超时时间
        _locationManager.locationTimeout = 10;
        // 设置获取地址信息超时时间
        _locationManager.reGeocodeTimeout = 10;
        // 设置delegate
        _locationManager.delegate = self;
    }
    return _locationManager;
}

/// 定位开关按钮
- (BMKBackgroundLocTabView *)locTabView {
    if (!_locTabView) {
        __weak typeof(self) weakSelf = self;
        _locTabView = [[BMKBackgroundLocTabView alloc] initWithFrame:CGRectMake(0, 0, self.view.frame.size.width, 60)];
        _locTabView.didClickTabBlock = ^(BOOL isSelected) {
            if (isSelected) { // 开始定位
                weakSelf.locInfoView.locInfo = nil;
                [weakSelf.locationManager startUpdatingLocation];
            } else { // 停止定位
                [weakSelf.locationManager stopUpdatingLocation];
            }
        };
    }
    return _locTabView;
}

/// 定位结果回调信息显示
- (BMKBackgroundLocInfoView *)locInfoView {
    if (!_locInfoView) {
        _locInfoView = [[BMKBackgroundLocInfoView alloc] initWithFrame:CGRectMake(10, CONTENT_HEIGHT - 200, self.view.frame.size.width - 20, 200)];
    }
    return _locInfoView;
}

@end
